package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.List;
import java.util.Objects;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class EntityNBTComponentImpl extends NBTComponentImpl<EntityNBTComponent, EntityNBTComponent.Builder> implements EntityNBTComponent {
   private final String selector;

   static EntityNBTComponent create(final @NotNull List<? extends ComponentLike> children, final @NotNull Style style, final String nbtPath, final boolean interpret, final @Nullable ComponentLike separator, final String selector) {
      return new EntityNBTComponentImpl(ComponentLike.asComponents(children, IS_NOT_EMPTY), (Style)Objects.requireNonNull(style, "style"), (String)Objects.requireNonNull(nbtPath, "nbtPath"), interpret, ComponentLike.unbox(separator), (String)Objects.requireNonNull(selector, "selector"));
   }

   EntityNBTComponentImpl(final @NotNull List<Component> children, final @NotNull Style style, final String nbtPath, final boolean interpret, final @Nullable Component separator, final String selector) {
      super(children, style, nbtPath, interpret, separator);
      this.selector = selector;
   }

   public @NotNull EntityNBTComponent nbtPath(final @NotNull String nbtPath) {
      return (EntityNBTComponent)(Objects.equals(this.nbtPath, nbtPath) ? this : create(this.children, this.style, nbtPath, this.interpret, this.separator, this.selector));
   }

   public @NotNull EntityNBTComponent interpret(final boolean interpret) {
      return (EntityNBTComponent)(this.interpret == interpret ? this : create(this.children, this.style, this.nbtPath, interpret, this.separator, this.selector));
   }

   public @Nullable Component separator() {
      return this.separator;
   }

   public @NotNull EntityNBTComponent separator(final @Nullable ComponentLike separator) {
      return create(this.children, this.style, this.nbtPath, this.interpret, separator, this.selector);
   }

   public @NotNull String selector() {
      return this.selector;
   }

   public @NotNull EntityNBTComponent selector(final @NotNull String selector) {
      return (EntityNBTComponent)(Objects.equals(this.selector, selector) ? this : create(this.children, this.style, this.nbtPath, this.interpret, this.separator, selector));
   }

   public @NotNull EntityNBTComponent children(final @NotNull List<? extends ComponentLike> children) {
      return create(children, this.style, this.nbtPath, this.interpret, this.separator, this.selector);
   }

   public @NotNull EntityNBTComponent style(final @NotNull Style style) {
      return create(this.children, style, this.nbtPath, this.interpret, this.separator, this.selector);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof EntityNBTComponent)) {
         return false;
      } else if (!super.equals(other)) {
         return false;
      } else {
         EntityNBTComponentImpl that = (EntityNBTComponentImpl)other;
         return Objects.equals(this.selector, that.selector());
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      result = 31 * result + this.selector.hashCode();
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   public EntityNBTComponent.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static final class BuilderImpl extends AbstractNBTComponentBuilder<EntityNBTComponent, EntityNBTComponent.Builder> implements EntityNBTComponent.Builder {
      private @Nullable String selector;

      BuilderImpl() {
      }

      BuilderImpl(final @NotNull EntityNBTComponent component) {
         super(component);
         this.selector = component.selector();
      }

      public EntityNBTComponent.@NotNull Builder selector(final @NotNull String selector) {
         this.selector = (String)Objects.requireNonNull(selector, "selector");
         return this;
      }

      public @NotNull EntityNBTComponent build() {
         if (this.nbtPath == null) {
            throw new IllegalStateException("nbt path must be set");
         } else if (this.selector == null) {
            throw new IllegalStateException("selector must be set");
         } else {
            return EntityNBTComponentImpl.create(this.children, this.buildStyle(), this.nbtPath, this.interpret, this.separator, this.selector);
         }
      }
   }
}
